package com.chelvc.framework.upload.support;

import java.awt.*;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.UUID;
import javax.imageio.ImageIO;

import com.chelvc.framework.base.util.FileUtils;
import com.chelvc.framework.upload.UploadHandler;
import com.chelvc.framework.upload.config.TencentUploadProperties;
import com.qcloud.cos.COSClient;
import com.qcloud.cos.model.ObjectMetadata;
import lombok.NonNull;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.condition.ConditionalOnBean;
import org.springframework.stereotype.Component;

/**
 * 基于腾讯COS文件上传实现
 *
 * @author Woody
 * @date 2023/4/5
 */
@Slf4j
@Component
@ConditionalOnBean(TencentUploadProperties.class)
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class TencentUploadHandler implements UploadHandler {
    private final COSClient client;
    private final TencentUploadProperties properties;

    /**
     * 将唯一标识转换成资源地址
     *
     * @param key 唯一标识
     * @return 资源地址
     */
    private String key2uri(String key) {
        return String.format(
                "https://%s.cos.%s.myqcloud.com/%s",
                this.properties.getBucket(), this.properties.getRegion(), key
        );
    }

    @Override
    public String upload(@NonNull File file) {
        if (log.isDebugEnabled()) {
            log.debug("Uploading file: {}", file);
        }
        String key = FileUtils.generateUniqueName(file);
        this.client.putObject(this.properties.getBucket(), key, file);
        return this.key2uri(key);
    }

    @Override
    public String upload(@NonNull InputStream stream, @NonNull String filename) {
        return this.upload(stream, filename, new ObjectMetadata());
    }

    @Override
    public String upload(@NonNull InputStream stream, @NonNull String filename, long size) {
        ObjectMetadata metadata = new ObjectMetadata();
        metadata.setContentLength(size);
        return this.upload(stream, filename, metadata);
    }

    @Override
    public String upload(@NonNull InputStream stream, @NonNull String filename, @NonNull ObjectMetadata metadata) {
        if (log.isDebugEnabled()) {
            log.debug("Uploading stream: {}", filename);
        }
        String key = FileUtils.generateUniqueName(filename);
        this.client.putObject(this.properties.getBucket(), key, stream, metadata);
        return this.key2uri(key);
    }

    @Override
    public String upload(@NonNull BufferedImage image) throws IOException {
        // 图片压缩
        int width = image.getWidth(null);
        int height = image.getHeight(null);
        BufferedImage compress = new BufferedImage(width, height, BufferedImage.TYPE_INT_RGB);
        compress.getGraphics().drawImage(image.getScaledInstance(width, height, Image.SCALE_SMOOTH), 0, 0, null);
        File target = new File(FileUtils.getUploadDirectory(), UUID.randomUUID().toString() + ".jpeg");
        try (FileOutputStream output = new FileOutputStream(target)) {
            ImageIO.write(compress, "jpeg", output);
        }

        // 上传图片
        try {
            return this.upload(target);
        } finally {
            FileUtils.delete(target);
        }
    }
}
